
#include <windows.h>
#include <stdio.h>

// disable warnings
#pragma warning(disable : 4996) // deprecation

// include
#include "Everything_ipc.h"

// error codes
#define EVERYTHING_OK						0
#define EVERYTHING_ERROR_MEMORY				1
#define EVERYTHING_ERROR_IPC				2
#define EVERYTHING_ERROR_REGISTERCLASSEX	3
#define EVERYTHING_ERROR_CREATEWINDOW		4
#define EVERYTHING_ERROR_CREATETHREAD		5
#define EVERYTHING_ERROR_INVALIDINDEX		6
#define EVERYTHING_ERROR_INVALIDCALL		7

// return copydata code
#define EVERYTHING_COPYDATA_QUERYCOMPLETEA	0

// internal state
char *_Everything_Search=0;
int _Everything_MatchPath=0;
int _Everything_MatchCase=0;
int _Everything_MatchWholeWord=0;
int _Everything_Regex=0;
int _Everything_LastError=0;
int _Everything_Max=EVERYTHING_IPC_ALLRESULTS;
int _Everything_Offset=0;
int _Everything_Detail=0;

EVERYTHING_IPC_LISTW *_Everything_List=0;

typedef struct
{
	char FullPathFileName[260];
	
	DWORD PathPartLength;
	DWORD FileNamePartOffset;

	DWORD SizeHigh;
	DWORD SizeLow;
	
	DWORD Attributes;
	/*
	DWORD CreationDate;
	DWORD CreationTime;
	
	DWORD LastAccessDate;
	DWORD LastAccessTime;
	
	DWORD LastWritDate;
	DWORD LastWritTime;
*/
	DWORD CreationYear;  
	DWORD CreationMonth;  
	DWORD CreationDayOfWeek;  
	DWORD CreationDay;  
	DWORD CreationHour;  
	DWORD CreationMinute;  
	DWORD CreationSecond;  
	DWORD CreationMilliseconds;

	DWORD LastAccessYear;  
	DWORD LastAccessMonth;  
	DWORD LastAccessDayOfWeek;  
	DWORD LastAccessDay;  
	DWORD LastAccessHour;  
	DWORD LastAccessMinute;  
	DWORD LastAccessSecond;  
	DWORD LastAccessMilliseconds;
	
	DWORD LastWriteYear;  
	DWORD LastWriteMonth;  
	DWORD LastWriteDayOfWeek;  
	DWORD LastWriteDay;  
	DWORD LastWriteHour;  
	DWORD LastWriteMinute;  
	DWORD LastWriteSecond;  
	DWORD LastWriteMilliseconds;
	
}EVERYTHING_RESULT;

// avoid other libs
int _Everything_StringLength(const char *start)
{
	register const char *s;
	
	s = start;
	
	while(*s)
	{
		s++;
	}
	
	return (int)(s-start);
}

void WINAPI Everything_SetSearch(const char *s)
{
	int len;
	
	if (_Everything_Search)
	{
		HeapFree(GetProcessHeap(),0,_Everything_Search);
		
		_Everything_Search = 0;
	}
	
	len = _Everything_StringLength(s) + 1;

	_Everything_Search = (char *)HeapAlloc(GetProcessHeap(),0,len*sizeof(char));
	if (_Everything_Search)
	{
		CopyMemory(_Everything_Search,s,len*sizeof(char));
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_MEMORY;
	}
}

void WINAPI Everything_SetMatchPath(int m)
{
	_Everything_MatchPath = m;
}

void WINAPI Everything_SetMatchCase(int m)
{
	_Everything_MatchCase = m;
}

void WINAPI Everything_SetMatchWholeWord(int m)
{
	_Everything_MatchWholeWord = m;
}

void WINAPI Everything_EnableRegex(int m)
{
	_Everything_Regex = m;
}

void WINAPI Everything_SetMax(int m)
{
	_Everything_Max = m;
}

void WINAPI Everything_SetOffset(int m)
{
	_Everything_Offset = m;
}
	
// custom window proc
LRESULT __stdcall window_proc(HWND hwnd,UINT msg,WPARAM wParam,LPARAM lParam)
{
	switch(msg)
	{
		case WM_COPYDATA:
		{
			COPYDATASTRUCT *cds = (COPYDATASTRUCT *)lParam;
			
			switch(cds->dwData)
			{
				case EVERYTHING_COPYDATA_QUERYCOMPLETEA:
				
					if (_Everything_List)
					{
						HeapFree(GetProcessHeap(),0,_Everything_List);
						
						_Everything_List = 0;
					}
					
					_Everything_List = (EVERYTHING_IPC_LISTW *)HeapAlloc(GetProcessHeap(),0,cds->cbData);
					
					if (_Everything_List)
					{
						CopyMemory(_Everything_List,cds->lpData,cds->cbData);
					}
					else
					{
						_Everything_LastError = EVERYTHING_ERROR_MEMORY;
					}
					
					PostQuitMessage(0);

					return TRUE;
			}
			
			break;
		}
	}
	
	return DefWindowProc(hwnd,msg,wParam,lParam);
}

DWORD __stdcall _Everything_thread_proc(void *param)
{
	EVERYTHING_IPC_QUERYA *query;
	HWND everything_hwnd;
	COPYDATASTRUCT cds;
	WNDCLASSEX wcex;
	HWND hwnd;
	MSG msg;
	int ret;
	int len;
	int size;
	
	ZeroMemory(&wcex,sizeof(wcex));
	wcex.cbSize = sizeof(wcex);
	
	if (!GetClassInfoEx(GetModuleHandle(0),TEXT("CLARION_EVERYTHING"),&wcex))
	{
		ZeroMemory(&wcex,sizeof(wcex));
		wcex.cbSize = sizeof(wcex);
		wcex.hInstance = GetModuleHandle(0);
		wcex.lpfnWndProc = window_proc;
		wcex.lpszClassName = TEXT("CLARION_EVERYTHING");
		
		if (!RegisterClassEx(&wcex))
		{
			_Everything_LastError = EVERYTHING_ERROR_REGISTERCLASSEX;
			
			return 0;
		}
	}
	
	hwnd = CreateWindow(
		TEXT("CLARION_EVERYTHING"),
		TEXT(""),
		0,
		0,0,0,0,
		0,0,GetModuleHandle(0),0);
		
	if (hwnd)
	{
		everything_hwnd = FindWindow(EVERYTHING_IPC_WNDCLASS,0);
		if (everything_hwnd)
		{
			len = _Everything_StringLength(_Everything_Search?_Everything_Search:"");
			
			size = sizeof(EVERYTHING_IPC_QUERYA) - sizeof(char) + len*sizeof(char) + sizeof(char);
			
			query = (EVERYTHING_IPC_QUERYA *)HeapAlloc(GetProcessHeap(),0,size);
			if (query)
			{
				query->max_results = _Everything_Max;
				query->offset = _Everything_Offset;
				query->reply_copydata_message = EVERYTHING_COPYDATA_QUERYCOMPLETEA;
				query->search_flags = (_Everything_Regex?EVERYTHING_IPC_REGEX:0) | (_Everything_MatchCase?EVERYTHING_IPC_MATCHCASE:0) | (_Everything_MatchWholeWord?EVERYTHING_IPC_MATCHWHOLEWORD:0) | (_Everything_MatchPath?EVERYTHING_IPC_MATCHPATH:0);
				query->reply_hwnd = hwnd;
				
				CopyMemory(query->search_string,_Everything_Search?_Everything_Search:"",(len+1)*sizeof(char));

				cds.cbData = size;
				cds.dwData = EVERYTHING_IPC_COPYDATAQUERYA;
				cds.lpData = query;
			
				if (SendMessage(everything_hwnd,WM_COPYDATA,(WPARAM)hwnd,(LPARAM)&cds) == TRUE)
				{
					// message pump
	loop:

					WaitMessage();
					
					// update windows
					while(PeekMessage(&msg,NULL,0,0,0)) 
					{
						ret = (int)GetMessage(&msg,0,0,0);
						if (ret == -1) goto exit;
						if (!ret) goto exit;
						
						// let windows handle it.
						TranslateMessage(&msg);
						DispatchMessage(&msg);
					}			
					
					goto loop;

	exit:
					
					// get result from window.
					DestroyWindow(hwnd);
				}
				else
				{
					_Everything_LastError = EVERYTHING_ERROR_IPC;
				}
				
				// get result from window.
				HeapFree(GetProcessHeap(),0,query);
			}
			else
			{
				_Everything_LastError = EVERYTHING_ERROR_MEMORY;
			}
		}
		else
		{
			// the everything window was not found.
			// we can optionally RegisterWindowMessage("EVERYTHING_IPC_CREATED") and 
			// wait for Everything to post this message to all top level windows when its up and running.
			_Everything_LastError = EVERYTHING_ERROR_IPC;
		}
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_CREATEWINDOW;
	}

	return 0;
}

void WINAPI Everything_Query(void)
{
	HANDLE hthread;
	DWORD threadid;
	
	hthread = CreateThread(0,0,_Everything_thread_proc,0,0,&threadid);
		
	if (hthread)
	{
		WaitForSingleObject(hthread,INFINITE);
		
		CloseHandle(hthread);
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_CREATETHREAD;
	}
}

void WINAPI Everything_SetDetail(int detail)
{
	_Everything_Detail = detail;
}

int _Everything_Compare(void *list,const void *a,const void *b)
{
	int i;
	
	i = stricmp(EVERYTHING_IPC_ITEMPATH(list,a),EVERYTHING_IPC_ITEMPATH(list,b));
	
	if (!i)
	{
		return stricmp(EVERYTHING_IPC_ITEMFILENAME(list,a),EVERYTHING_IPC_ITEMFILENAME(list,b));
	}
	else
	if (i > 0)
	{
		return 1;
	}
	else
	{
		return -1;
	}
}
	
int _Everything_NoDriveLetterCompare(void *list,const void *a,const void *b)
{
	int i;
	
//	i = stricmp(EVERYTHING_IPC_ITEMPATH(list,a),EVERYTHING_IPC_ITEMPATH(list,b));
	if (EVERYTHING_IPC_ITEMPATH(list,a)[0])
	{
		if (EVERYTHING_IPC_ITEMPATH(list,b)[0])
		{
			i = stricmp(EVERYTHING_IPC_ITEMPATH(list,a)+1,EVERYTHING_IPC_ITEMPATH(list,b)+1);
		}
		else
		{
			i = stricmp(EVERYTHING_IPC_ITEMPATH(list,a)+1,EVERYTHING_IPC_ITEMPATH(list,b));
		}
	}
	else
	{
		if (EVERYTHING_IPC_ITEMPATH(list,b)[0])
		{
			i = stricmp(EVERYTHING_IPC_ITEMPATH(list,a),EVERYTHING_IPC_ITEMPATH(list,b)+1);
		}
		else
		{
			i = stricmp(EVERYTHING_IPC_ITEMPATH(list,a),EVERYTHING_IPC_ITEMPATH(list,b));
		}
	}
	
	if (!i)
	{
		return stricmp(EVERYTHING_IPC_ITEMFILENAME(list,a),EVERYTHING_IPC_ITEMFILENAME(list,b));
	}
	else
	if (i > 0)
	{
		return 1;
	}
	else
	{
		return -1;
	}
}
	
void WINAPI Everything_SortResults(int NoDriveLetter)
{
	if (_Everything_List)
	{
		if (NoDriveLetter)
		{
			qsort_s(_Everything_List->items,_Everything_List->numitems,sizeof(EVERYTHING_IPC_ITEM),_Everything_NoDriveLetterCompare,_Everything_List);
		}
		else
		{
			qsort_s(_Everything_List->items,_Everything_List->numitems,sizeof(EVERYTHING_IPC_ITEM),_Everything_Compare,_Everything_List);
		}
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;
	}
}

int WINAPI Everything_GetLastError(void)
{
	return _Everything_LastError;
}

int WINAPI Everything_GetNumFileResults(void)
{
	if (_Everything_List)
	{
		return _Everything_List->numfiles;
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;

		return 0;
	}
}

int WINAPI Everything_GetNumFolderResults(void)
{
	if (_Everything_List)
	{
		return _Everything_List->numfolders;
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;

		return 0;
	}
}

int WINAPI Everything_GetNumResults(void)
{
	if (_Everything_List)
	{
		return _Everything_List->numitems;
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;

		return 0;
	}
}

int WINAPI Everything_GetTotFileResults(void)
{
	if (_Everything_List)
	{
		return _Everything_List->totfiles;
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;
		
		return 0;
	}
}

int WINAPI Everything_GetTotFolderResults(void)
{
	if (_Everything_List)
	{
		return _Everything_List->totfolders;
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;

		return 0;
	}
}

int WINAPI Everything_GetTotResults(void)
{
	if (_Everything_List)
	{
		return _Everything_List->totitems;
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;

		return 0;
	}
}

int WINAPI Everything_IsVolumeResult(int index)
{
	if (_Everything_List)
	{
		if (index < 0)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return 0;
		}
		
		if (index >= (int)_Everything_List->numitems)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return 0;
		}
		
		return _Everything_List->items[index].flags & EVERYTHING_IPC_DRIVE;
	}
	else
	{
		return 0;
	}
}

int WINAPI Everything_IsFolderResult(int index)
{
	if (_Everything_List)
	{
		if (index < 0)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return 0;
		}
		
		if (index >= (int)_Everything_List->numitems)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return 0;
		}
		
		return _Everything_List->items[index].flags & (EVERYTHING_IPC_DRIVE|EVERYTHING_IPC_FOLDER);
	}
	else
	{
		return 0;
	}
}

int WINAPI Everything_IsFileResult(int index)
{
	if (_Everything_List)
	{
		if (index < 0)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return 0;
		}
		
		if (index >= (int)_Everything_List->numitems)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return 0;
		}
		
		return !(_Everything_List->items[index].flags & (EVERYTHING_IPC_DRIVE|EVERYTHING_IPC_FOLDER));
	}
	else
	{
		return 0;
	}
}

const char *WINAPI Everything_GetResultFileName(int index)
{
	if (_Everything_List)
	{
		if (index < 0)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return "";
		}
		
		if (index >= (int)_Everything_List->numitems)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return "";
		}
		
		return EVERYTHING_IPC_ITEMFILENAMEA(_Everything_List,&_Everything_List->items[index]);
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;

		return "";
	}
}

const char *WINAPI Everything_GetResultPath(int index)
{
	if (_Everything_List)
	{
		if (index < 0)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return "";
		}
		
		if (index >= (int)_Everything_List->numitems)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return "";
		}
		
		return EVERYTHING_IPC_ITEMPATHA(_Everything_List,&_Everything_List->items[index]);
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;

		return "";
	}
}

// max is in chars
static int _Everything_Copy(char *buf,int max,int catlen,const char *s)
{
	int len;
	
	buf += catlen;
	max -= catlen;
	
	len = _Everything_StringLength(s);
	if (!len) return catlen;

	// terminate
	if (len > max-1) len = max-1;

	CopyMemory(buf,s,len*sizeof(char));

	buf[len] = 0;
	
	return len + catlen;

}

void WINAPI Everything_GetResult(int index,EVERYTHING_RESULT *result)
{
	WIN32_FIND_DATA fd;
	SYSTEMTIME st;
	int len;
	
	if (!result)
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
		
		return;
	}

	if (_Everything_List)
	{
		if (index < 0)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return;
		}
		
		if (index >= (int)_Everything_List->numitems)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return;
		}
		
		len = _Everything_Copy(result->FullPathFileName,260,0,EVERYTHING_IPC_ITEMPATHA(_Everything_List,&_Everything_List->items[index]));
		result->PathPartLength = len;
		 
		if (_Everything_StringLength(EVERYTHING_IPC_ITEMPATHA(_Everything_List,&_Everything_List->items[index]))) 
		{
			len = _Everything_Copy(result->FullPathFileName,260,len,"\\");
		}

		result->FileNamePartOffset = len;
		
		_Everything_Copy(result->FullPathFileName,260,len,EVERYTHING_IPC_ITEMFILENAMEA(_Everything_List,&_Everything_List->items[index]));
		
		if (_Everything_Detail)
		{
			GetFileAttributesEx(result->FullPathFileName,GetFileExInfoStandard,&fd);
			
			result->SizeLow = fd.nFileSizeLow;
			result->SizeHigh = fd.nFileSizeHigh;
			result->Attributes = fd.dwFileAttributes;
			
			FileTimeToSystemTime(&fd.ftCreationTime,&st);
			result->CreationDay = st.wDay;
			result->CreationDayOfWeek = st.wDayOfWeek;
			result->CreationHour = st.wHour;
			result->CreationMilliseconds = st.wMilliseconds;
			result->CreationMinute = st.wMinute;
			result->CreationMonth = st.wMonth;
			result->CreationSecond = st.wSecond;
			result->CreationYear = st.wYear;
			
			FileTimeToSystemTime(&fd.ftLastAccessTime,&st);
			result->LastAccessDay = st.wDay;
			result->LastAccessDayOfWeek = st.wDayOfWeek;
			result->LastAccessHour = st.wHour;
			result->LastAccessMilliseconds = st.wMilliseconds;
			result->LastAccessMinute = st.wMinute;
			result->LastAccessMonth = st.wMonth;
			result->LastAccessSecond = st.wSecond;
			result->LastAccessYear = st.wYear;
			
			FileTimeToSystemTime(&fd.ftLastWriteTime,&st);
			result->LastWriteDay = st.wDay;
			result->LastWriteDayOfWeek = st.wDayOfWeek;
			result->LastWriteHour = st.wHour;
			result->LastWriteMilliseconds = st.wMilliseconds;
			result->LastWriteMinute = st.wMinute;
			result->LastWriteMonth = st.wMonth;
			result->LastWriteSecond = st.wSecond;
			result->LastWriteYear = st.wYear;
		}
		else
		{
			result->Attributes = 0;
			
			if (_Everything_List->items[index].flags & EVERYTHING_IPC_DRIVE) result->Attributes = FILE_ATTRIBUTE_DIRECTORY;
			if (_Everything_List->items[index].flags & EVERYTHING_IPC_FOLDER) result->Attributes = FILE_ATTRIBUTE_DIRECTORY;
		}
		
		return;
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;

		return;
	}	
}
/*
EVERYTHING_RESULT * WINAPI Everything_CreateResultArray(void)
{
	WIN32_FIND_DATA fd;
	SYSTEMTIME st;
	int len;
	
	if (!result)
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
		
		return;
	}

	if (_Everything_List)
	{
		if (index < 0)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return;
		}
		
		if (index >= (int)_Everything_List->numitems)
		{
			_Everything_LastError = EVERYTHING_ERROR_INVALIDINDEX;
			
			return;
		}
		
		len = _Everything_Copy(result->FullPathFileName,260,0,EVERYTHING_IPC_ITEMPATHA(_Everything_List,&_Everything_List->items[index]));
		result->PathPartLength = len;
		 
		if (_Everything_StringLength(EVERYTHING_IPC_ITEMPATHA(_Everything_List,&_Everything_List->items[index]))) 
		{
			len = _Everything_Copy(result->FullPathFileName,260,len,"\\");
		}

		result->FileNamePartOffset = len;
		
		_Everything_Copy(result->FullPathFileName,260,len,EVERYTHING_IPC_ITEMFILENAMEA(_Everything_List,&_Everything_List->items[index]));
		
		if (_Everything_Detail)
		{
			GetFileAttributesEx(result->FullPathFileName,GetFileExInfoStandard,&fd);
			
			result->SizeLow = fd.nFileSizeLow;
			result->SizeHigh = fd.nFileSizeHigh;
			result->Attributes = fd.dwFileAttributes;
			
			FileTimeToSystemTime(&fd.ftCreationTime,&st);
			result->CreationDay = st.wDay;
			result->CreationDayOfWeek = st.wDayOfWeek;
			result->CreationHour = st.wHour;
			result->CreationMilliseconds = st.wMilliseconds;
			result->CreationMinute = st.wMinute;
			result->CreationMonth = st.wMonth;
			result->CreationSecond = st.wSecond;
			result->CreationYear = st.wYear;
			
			FileTimeToSystemTime(&fd.ftLastAccessTime,&st);
			result->LastAccessDay = st.wDay;
			result->LastAccessDayOfWeek = st.wDayOfWeek;
			result->LastAccessHour = st.wHour;
			result->LastAccessMilliseconds = st.wMilliseconds;
			result->LastAccessMinute = st.wMinute;
			result->LastAccessMonth = st.wMonth;
			result->LastAccessSecond = st.wSecond;
			result->LastAccessYear = st.wYear;
			
			FileTimeToSystemTime(&fd.ftLastWriteTime,&st);
			result->LastWriteDay = st.wDay;
			result->LastWriteDayOfWeek = st.wDayOfWeek;
			result->LastWriteHour = st.wHour;
			result->LastWriteMilliseconds = st.wMilliseconds;
			result->LastWriteMinute = st.wMinute;
			result->LastWriteMonth = st.wMonth;
			result->LastWriteSecond = st.wSecond;
			result->LastWriteYear = st.wYear;
		}
		else
		{
			result->Attributes = 0;
			
			if (_Everything_List->items[index].flags & EVERYTHING_IPC_DRIVE) result->Attributes = FILE_ATTRIBUTE_DIRECTORY;
			if (_Everything_List->items[index].flags & EVERYTHING_IPC_FOLDER) result->Attributes = FILE_ATTRIBUTE_DIRECTORY;
		}
		
		return;
	}
	else
	{
		_Everything_LastError = EVERYTHING_ERROR_INVALIDCALL;

		return;
	}	
}
*/
//void DestroyResultArray(void *)

// testing

int main(int,wchar_t**)
{
	EVERYTHING_RESULT result;
	
	Everything_SetSearch("foo.bmp");
	Everything_Query();

	Everything_GetResult(0,&result);

	MessageBox(0,result.FullPathFileName,"result 1",MB_OK);
}

/*
EVERYTHING_RESULT GROUP

	FullPathFileName	STRING(260)
	
	PathLen				LONG
	FileNamePart		*CSTRING

	SizeHigh			ULONG
	SizeLow				ULONG
	
	Attributes			ULONG

	CreationYear		ULONG ! The year (1601 - 30827). 
	CreationMonth		ULONG ! The month (January = 1, December = 12)
	CreationDayOfWeek	ULONG ! The day of the week. (sunday = 0, monday = 1, Saturday = 6)
	CreationDay		ULONG ! The day of the month (1-31). 
	CreationHour		ULONG ! The hour (0-23).
	CreationMinute		ULONG ! The minute (0-59). 
	CreationSecond		ULONG ! The second (0-59). 
	CreationMilliseconds	ULONG ! The millisecond (0-999). 

	CreationYear		ULONG ! The year (1601 - 30827). 
	CreationMonth		ULONG ! The month (January = 1, December = 12)
	CreationDayOfWeek	ULONG ! The day of the week. (sunday = 0, monday = 1, Saturday = 6)
	CreationDay			ULONG ! The day of the month (1-31). 
	CreationHour		ULONG ! The hour (0-23).
	CreationMinute		ULONG ! The minute (0-59). 
	CreationSecond		ULONG ! The second (0-59). 
	CreationMilliseconds	ULONG ! The millisecond (0-999). 

	LastAccessYear		ULONG ! The year (1601 - 30827). 
	LastAccessMonth		ULONG ! The month (January = 1, December = 12)
	LastAccessDayOfWeek	ULONG ! The day of the week. (sunday = 0, monday = 1, Saturday = 6)
	LastAccessDay		ULONG ! The day of the month (1-31). 
	LastAccessHour		ULONG ! The hour (0-23).
	LastAccessMinute	ULONG ! The minute (0-59). 
	LastAccessSecond	ULONG ! The second (0-59). 
	LastAccessMilliseconds	ULONG ! The millisecond (0-999). 
	
	LastWriteYear		ULONG ! The year (1601 - 30827). 
	LastWriteMonth		ULONG ! The month (January = 1, December = 12)
	LastWriteDayOfWeek	ULONG ! The day of the week. (sunday = 0, monday = 1, Saturday = 6)
	LastWriteDay		ULONG ! The day of the month (1-31). 
	LastWriteHour		ULONG ! The hour (0-23).
	LastWriteMinute		ULONG ! The minute (0-59). 
	LastWriteSecond		ULONG ! The second (0-59). 
	LastWriteMilliseconds	ULONG ! The millisecond (0-999). 
	
END 
*/